package com.josyf.commandbuttons;

import com.alibaba.fastjson.JSON;
import com.cedarsoftware.util.io.JsonWriter;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;

import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class ConfigFile {

    private static final JSONParser parser = new JSONParser();
    private static final String CONFIG_FILE_NAME = "commands1.json"; // 改下名称防止冲突，不过我也不确定会不会有人同时加俩版本

    // Read commands1.json, convert it to an array, and append A JSON OBJECT
    public static void appendToFile(JSONObject jsonObject) {
        JSONArray array = getJsonArrayFromFile();
        if (array == null) {
            array = new JSONArray();
        }
        array.add(jsonObject);
        writeToFile(array);
    }

    // 按名称
    public static void removeObjectByName(String name) {
        JSONArray array = getJsonArrayFromFile();
        if (array == null) {
            return;
        }
        JSONArray newArray = new JSONArray();
        boolean removed = false;
        for (Object obj : array) {
            if (obj instanceof JSONObject) {
                JSONObject jsonObject = (JSONObject) obj;
                if (jsonObject.containsKey("name") && jsonObject.get("name").equals(name)) {
                    removed = true;
                } else {
                    newArray.add(jsonObject);
                }
            }
        }
        if (removed) {
            writeToFile(newArray);
        }
    }

    public static void updateObjectByName(String originalName, JSONObject updatedObject) {
        JSONArray array = getJsonArrayFromFile();
        if (array == null) {
            return;
        }
        JSONArray newArray = new JSONArray();
        boolean updated = false;
        for (Object obj : array) {
            if (obj instanceof JSONObject) {
                JSONObject jsonObject = (JSONObject) obj;
                if (jsonObject.containsKey("name") && jsonObject.get("name").equals(originalName)) {
                    newArray.add(updatedObject);
                    updated = true;
                } else {
                    newArray.add(jsonObject);
                }
            }
        }
        if (updated) {
            writeToFile(newArray);
        } else {
            System.err.println("未找到名称为'" + originalName + "'的命令以进行更新。");
        }
        writeToFile(newArray);
    }


    // overwrites current commands1.json w/ updated jsonArray
    private static void writeToFile(JSONArray jsonArray) {
        FileWriter fileWriter = null;
        try {
            fileWriter = new FileWriter(CONFIG_FILE_NAME);
            // Use fastjson for writing as it handles JSONArray better with JsonWriter formatting
            String jArrayToString = JSON.toJSONString(jsonArray);
            String formattedJson = JsonWriter.formatJson(jArrayToString);
            fileWriter.write(formattedJson);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (fileWriter != null) {
                    fileWriter.flush();
                    fileWriter.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    // parses commands1.json into JSONArray
    private static JSONArray getJsonArrayFromFile() {
        try (FileReader reader = new FileReader(CONFIG_FILE_NAME)) {
            Object obj = parser.parse(reader);
            if (obj instanceof JSONArray) {
                return (JSONArray) obj;
            } else if (obj instanceof JSONObject) {

                System.err.println("配置文件不是一个 JSON 数组。返回空数组。");
                return new JSONArray();
            } else {
                System.err.println("配置文件包含意外的内容。返回空数组。");
                return new JSONArray();
            }
        } catch (IOException e) {
            System.out.println("commands1.json文件不存在或读取错误。返回null。");
            return null;
        } catch (ParseException e) {
            System.err.println("解析 commands1.json 时出错:" + e.getMessage());
            e.printStackTrace();
            return new JSONArray();
        }
    }


    // parses commands1.json into
    public static ArrayList<JSONObject> getArrayFromJsonFile() {
        JSONArray array = getJsonArrayFromFile();
        if (array == null) {
            return null;
        }

        ArrayList<JSONObject> commandObjects = new ArrayList<>();
        for (Object obj : array) {
            if (obj instanceof JSONObject) {
                commandObjects.add((JSONObject) obj);
            }
        }
        return commandObjects;
    }

    // 添加
    public static void addObject(JSONObject jsonObject) {
        ArrayList<JSONObject> commListCopy = CommandButtons.getMasterCommList();
        if (commListCopy == null) {
            commListCopy = new ArrayList<>();
            CommandButtons.setMasterCommList(commListCopy);
        }
        commListCopy.add(jsonObject);
        appendToFile(jsonObject);
    }

    // 移除
    public static void removeObject(JSONObject objToRemove) {
        ArrayList<JSONObject> commListCopy = CommandButtons.getMasterCommList();
        if (commListCopy != null) {
            commListCopy.removeIf(obj -> obj.containsKey("name") && objToRemove.containsKey("name") && obj.get("name").equals(objToRemove.get("name")));
        }
        if (objToRemove.containsKey("name")) {
            removeObjectByName(objToRemove.get("name").toString());
        } else {
            System.err.println("无法删除没有“name”字段的对象。");
        }
    }


    // 编辑
    public static void updateObject(String originalName, JSONObject updatedObject) {
        ArrayList<JSONObject> commListCopy = CommandButtons.getMasterCommList();
        if (commListCopy != null) {
            for (int i = 0; i < commListCopy.size(); i++) {
                JSONObject obj = commListCopy.get(i);
                if (obj.containsKey("name") && obj.get("name").equals(originalName)) {
                    commListCopy.set(i, updatedObject);
                    break;
                }
            }
        }
        updateObjectByName(originalName, updatedObject);
    }
}
