package com.josyf.commandbuttons.gui;

import com.josyf.commandbuttons.CommandButtons;
import com.josyf.commandbuttons.ConfigFile;
import io.github.cottonmc.cotton.gui.client.BackgroundPainter;
import io.github.cottonmc.cotton.gui.client.LightweightGuiDescription;
import io.github.cottonmc.cotton.gui.widget.*;
import io.github.cottonmc.cotton.gui.widget.data.HorizontalAlignment;
import io.github.cottonmc.cotton.gui.widget.data.VerticalAlignment;
import net.minecraft.client.MinecraftClient;
import io.github.cottonmc.cotton.gui.widget.data.Insets;
import net.minecraft.client.gui.screen.Screen;
import net.minecraft.text.Text;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;

import java.util.ArrayList;
import java.util.List;

public class ButtonGUI extends LightweightGuiDescription {

    private final Screen parentScreen;

    public ButtonGUI() {
        this.parentScreen = MinecraftClient.getInstance().currentScreen;
        // initialize root panel of GUI
        WGridPanel root = new WGridPanel();
        setRootPanel(root);
        root.setSize(300, 200);


        root.setInsets(new Insets(5));

        setupBackground(root);

        WLabel title = new WLabel(Text.translatable("gui.commandbuttons.title"), 0xcdcdcd);
        title.setHorizontalAlignment(HorizontalAlignment.CENTER);
        root.add(title, 0, 0, 18, 1);

        addCloseButton(root);

        WGridPanel commandListPanel = new WGridPanel();
        WScrollPanel scrollPanel = new WScrollPanel(commandListPanel);
        root.add(scrollPanel, 0, 2, 18, 8);

        addSavedButtons(commandListPanel);

        WButton addCommandButton = new WButton(Text.translatable("gui.commandbuttons.add_command"));
        addCommandButton.setOnClick(this::openCreateCommandScreen);
        root.add(addCommandButton, 0, 11, 18, 1);

        root.validate(this);
    }

    // 背景
    private void setupBackground(WGridPanel root) {
        root.setBackgroundPainter(BackgroundPainter.createColorful(0x4D000000));
    }

    // 关闭按钮
    private void addCloseButton(WGridPanel root) {
        WButton escButton = new WButton(Text.translatable("gui.commandbuttons.close"));
        escButton.setOnClick(() -> MinecraftClient.getInstance().setScreen(this.parentScreen));
        root.add(escButton, 17, 0, 1, 1);
    }

    // 保存的设置
    private void addSavedButtons(WGridPanel commandListPanel) {
        ArrayList<JSONObject> commListCopy = CommandButtons.getMasterCommList();
        if (commListCopy != null) {
            int yOffset = 0;
            for (JSONObject object : commListCopy) {
                if (object.containsKey("name") && object.containsKey("commands")) {
                    String name = object.get("name").toString();
                    JSONArray commandsArray = (JSONArray) object.get("commands");
                    List<String> commands = new ArrayList<>();
                    for (Object cmdObj : commandsArray) {
                        if (cmdObj instanceof String) {
                            commands.add((String) cmdObj);
                        }
                    }

                    WGridPanel commandEntryPanel = new WGridPanel();
                    commandEntryPanel.setSize(commandListPanel.getWidth(), 20);

                    WButton executeButton = new WButton(Text.literal(name));
                    executeButton.setOnClick(() -> {
                        CommandButtons.runCommands(commands);
                        MinecraftClient.getInstance().setScreen(this.parentScreen);
                    });
                    commandEntryPanel.add(executeButton, 0, 0, 14, 1);

                    WButton editButton = new WButton(Text.translatable("gui.commandbuttons.edit"));
                    editButton.setOnClick(() -> openEditCommandScreen(object));
                    commandEntryPanel.add(editButton, 15, 0, 3, 1);

                    commandListPanel.add(commandEntryPanel, 0, yOffset, 18, 1);
                    yOffset += 1;
                } else {
                    System.err.println("在 commands1.json 中跳过格式错误的命令对象:" + object.toJSONString());
                }
            }
        }
    }

    // 新建按钮
    private void openCreateCommandScreen() {
        MinecraftClient.getInstance().setScreen(new ButtonGUIScreen(new CreateCommandGUI(this.parentScreen), this.parentScreen));
    }

    // 编辑按钮
    private void openEditCommandScreen(JSONObject commandObject) {
        MinecraftClient.getInstance().setScreen(new ButtonGUIScreen(new EditCommandGUI(commandObject, this.parentScreen), this.parentScreen));
    }


    @Override
    public void addPainters() {
    }

    @Override
    public boolean isTitleVisible() {
        return true;
    }

    @Override
    public int getTitleColor() {
        return 0x404040;
    }
}
